@@ -46,8 +46,12 @@ from mongoengine import (
     ObjectIdField,
     ReferenceField,
     StringField,
+    UUIDField,
+    ValidationError,
 )
-from mongoengine.errors import DoesNotExist
+
+from beer_garden import config
+from beer_garden.db.mongo.querysets import FileFieldHandlingQuerySet
 
 from .fields import DummyField, StatusInfo
 from .validators import validate_permissions
@@ -62,7 +66,7 @@ __all__ = [
     "Event",
     "Principal",
     "LegacyRole",
-    "RefreshToken",
+    "UserToken",
     "Job",
     "RequestTemplate",
     "DateTrigger",
@@ -75,6 +79,7 @@ __all__ = [
     "Role",
     "RoleAssignment",
     "User",
+    "CommandPublishingBlockList",
 ]
 
 REQUEST_MAX_PARAM_SIZE = 5 * 1_000_000
@@ -108,6 +113,14 @@ class MongoModel:
     def pre_serialize(self):
         pass
 
+    @property
+    def changed_fields(self):
+        return getattr(self, "_changed_fields", [])
+
+    @property
+    def created(self):
+        return getattr(self, "_created", False)
+
 
 # MongoEngine needs all EmbeddedDocuments to be defined before any Documents that
 # reference them. So Parameter must be defined before Command, and choices should be
@@ -311,6 +324,7 @@ class Request(MongoModel, Document):
     command_type = StringField(choices=BrewtilsCommand.COMMAND_TYPES)
     created_at = DateTimeField(default=datetime.datetime.utcnow, required=True)
     updated_at = DateTimeField(default=None, required=True)
+    status_updated_at = DateTimeField()
     error_class = StringField(required=False)
     has_parent = BooleanField(required=False)
     hidden = BooleanField(required=False)
@@ -318,6 +332,7 @@ class Request(MongoModel, Document):
     parameters_gridfs = FileField()
 
     meta = {
+        "queryset_class": FileFieldHandlingQuerySet,
         "auto_create_index": False,  # We need to manage this ourselves
         "index_background": True,
         "indexes": [
@@ -330,6 +345,7 @@ class Request(MongoModel, Document):
             {"name": "status_index", "fields": ["status"]},
             {"name": "created_at_index", "fields": ["created_at"]},
             {"name": "updated_at_index", "fields": ["updated_at"]},
+            {"name": "status_updated_at_index", "fields": ["status_updated_at"]},
             {"name": "comment_index", "fields": ["comment"]},
             {"name": "parent_ref_index", "fields": ["parent"]},
             {"name": "parent_index", "fields": ["has_parent"]},
@@ -418,26 +434,74 @@ class Request(MongoModel, Document):
             self.parameters = json.loads(self.parameters_gridfs.read().decode(encoding))
             self.parameters_gridfs = None
 
-    def save(self, *args, **kwargs):
-        """Save a request, moving request attributes to GridFS if too big"""
+    def _pre_save(self):
+        """Move request attributes to GridFS if too big"""
         self.updated_at = datetime.datetime.utcnow()
         encoding = "utf-8"
 
-        if self.parameters:
+        # NOTE: The following was added for #1216, which aims to resolve the duplication
+        # and orphaning of files in gridfs. It is less than ideal to do an additional
+        # database lookup, but the various conversions to and from brewtils mean that
+        # we get here having lost the parameters_gridfs and output_gridfs values,
+        # preventing us from checking if they've already been populated. Rather than
+        # perform a potentially dangerous rework of the entire Request update flow,
+        # we opt to just pull the Request as it exists in the database so that we can
+        # check those gridfs field.
+        if self.id:
+            try:
+                old_request = Request.objects.get(id=self.id)
+                self.parameters_gridfs = old_request.parameters_gridfs
+                self.output_gridfs = old_request.output_gridfs
+            except self.DoesNotExist:
+                # Requests to child gardens have an id set from the parent, but no
+                # local Request yet
+                pass
+
+        if self.parameters and self.parameters_gridfs.grid_id is None:
             params_json = json.dumps(self.parameters)
             if len(params_json) > REQUEST_MAX_PARAM_SIZE:
-                self.logger.debug("Parameters too bg, storing in GridFS")
+                self.logger.debug("Parameters too big, storing in GridFS")
                 self.parameters_gridfs.put(params_json, encoding=encoding)
-                self.parameters = None
 
-        if self.output:
+        if self.parameters_gridfs.grid_id:
+            self.parameters = None
+
+        if self.output and self.output_gridfs.grid_id is None:
             output_json = json.dumps(self.output)
             if len(output_json) > REQUEST_MAX_PARAM_SIZE:
                 self.logger.info("Output size too big, storing in gridfs")
-                self.output_gridfs.put(output_json, encoding=encoding)
-                self.output = None
+                self.output_gridfs.put(self.output, encoding=encoding)
 
+        if self.output_gridfs.grid_id:
+            self.output = None
+
+    def _post_save(self):
+        if self.status == "CREATED" and self.namespace == config.get("garden.name"):
+            self._update_raw_file_references()
+
+    def _update_raw_file_references(self):
+        parameters = self.parameters or {}
+
+        for param_value in parameters.values():
+            if (
+                isinstance(param_value, dict)
+                and param_value.get("type") == "bytes"
+                and param_value.get("id") is not None
+            ):
+                try:
+                    raw_file = RawFile.objects.get(id=param_value["id"])
+                    raw_file.request = self
+                    raw_file.save()
+                except RawFile.DoesNotExist:
+                    self.logger.debug(
+                        f"Error locating RawFile with id {param_value['id']} "
+                        "while saving Request {self.id}"
+                    )
+
+    def save(self, *args, **kwargs):
+        self._pre_save()
         super(Request, self).save(*args, **kwargs)
+        self._post_save()
 
     def clean(self):
         """Validate before saving to the database"""
@@ -473,6 +537,11 @@ class Request(MongoModel, Document):
                 f"consistent with has_parent value of {self.has_parent}"
             )
 
+        if (self.namespace == config.get("garden.name")) and (
+            "status" in self.changed_fields or self.created
+        ):
+            self.status_updated_at = datetime.datetime.utcnow()
+
     def clean_update(self):
         """Ensure that the update would not result in an illegal status transition"""
         # Get the original status
@@ -584,26 +653,6 @@ class Principal(MongoModel, Document):
     }
 
 
-class RefreshToken(Document):
-    brewtils_model = brewtils.models.RefreshToken
-
-    issued = DateTimeField(required=True)
-    expires = DateTimeField(required=True)
-    payload = DictField(required=True)
-
-    meta = {"indexes": [{"fields": ["expires"], "expireAfterSeconds": 0}]}
-
-    def get_principal(self):
-        principal_id = self.payload.get("sub")
-        if not principal_id:
-            return None
-
-        try:
-            return Principal.objects.get(id=principal_id)
-        except DoesNotExist:
-            return None
-
-
 class RequestTemplate(MongoModel, EmbeddedDocument):
     brewtils_model = brewtils.models.RequestTemplate
 
@@ -856,6 +905,20 @@ class FileChunk(MongoModel, Document):
 
 class RawFile(Document):
     file = FileField()
+    created_at = DateTimeField(default=datetime.datetime.utcnow, required=True)
+    request = LazyReferenceField(Request, required=False, reverse_delete_rule=CASCADE)
+
+    meta = {"queryset_class": FileFieldHandlingQuerySet}
+
+
+class CommandPublishingBlockList(Document):
+    namespace = StringField(required=True)
+    system = StringField(required=True)
+    command = StringField(required=True)
+
+    meta = {
+        "indexes": [{"fields": ["namespace", "system", "command"], "unique": True}],
+    }
 
 
 class Role(Document):
@@ -869,13 +932,19 @@ class Role(Document):
 
 
 class RoleAssignmentDomain(EmbeddedDocument):
-    scope = StringField(required=True, choices=["Garden", "System", "Command"])
-    identifiers = DictField(required=True)
+    scope = StringField(required=True, choices=["Garden", "Global", "System"])
+    identifiers = DictField(required=False)
+
+    def clean(self):
+        if self.identifiers == {} and self.scope != "Global":
+            raise ValidationError(
+                "identifiers field is required for all scopes other than Global"
+            )
 
 
 class RoleAssignment(EmbeddedDocument):
-    domain = EmbeddedDocumentField(RoleAssignmentDomain)
-    role = ReferenceField("Role")
+    domain = EmbeddedDocumentField(RoleAssignmentDomain, required=True)
+    role = ReferenceField("Role", required=True)
 
 
 class User(Document):
@@ -905,6 +974,16 @@ class User(Document):
 
         return self._permissions_cache
 
+    @property
+    def domain_permissions(self) -> dict:
+        """Returns the domain_permissions portion of self.permissions"""
+        return self.permissions["domain_permissions"]
+
+    @property
+    def global_permissions(self) -> dict:
+        """Returns the global_permissions portion of self.permissions"""
+        return self.permissions["global_permissions"]
+
     def clear_permissions_cache(self) -> None:
         """Clear the cached permission set for the user. This is useful if the user's
         role assignments have been changed and you want to perform a permission check
@@ -955,3 +1034,25 @@ class User(Document):
             bool: True if the password matches, False otherwise
         """
         return custom_app_context.verify(password, self.password)
+
+    def revoke_tokens(self) -> None:
+        """Remove all tokens from the user's list of valid tokens. This is useful for
+        requiring the user to explicitly login, which one may want to do for a variety
+        of reasons.
+        """
+        UserToken.objects.filter(user=self).delete()
+
+
+class UserToken(Document):
+    issued_at = DateTimeField(required=True, default=datetime.datetime.utcnow)
+    expires_at = DateTimeField(required=True)
+    user = LazyReferenceField("User", required=True, reverse_delete_rule=CASCADE)
+    uuid = UUIDField(binary=False, required=True, unique="True")
+
+    meta = {
+        "indexes": [
+            "user",
+            "uuid",
+            {"fields": ["expires_at"], "expireAfterSeconds": 0},
+        ]
+    }
