@@ -6,7 +6,7 @@ from typing import List, Tuple
 from brewtils.stoppable_thread import StoppableThread
 from mongoengine import Q
 
-from beer_garden.db.mongo.models import File, Request
+from beer_garden.db.mongo.models import File, RawFile, Request
 
 
 class MongoPruner(StoppableThread):
@@ -31,7 +31,9 @@ class MongoPruner(StoppableThread):
         )
 
     def run(self):
-        self.logger.debug(self.display_name + " is started")
+        self.logger.debug(
+            "%s is started with run interval %ss", self.display_name, self._run_every
+        )
 
         while not self.wait(self._run_every):
             current_time = datetime.utcnow()
@@ -114,6 +116,13 @@ class MongoPruner(StoppableThread):
                     ),
                 }
             )
+            prune_tasks.append(
+                {
+                    "collection": RawFile,
+                    "field": "created_at",
+                    "delete_after": timedelta(minutes=file_ttl),
+                }
+            )
 
         # Look at the various TTLs to determine how often to run
         real_ttls = [x for x in kwargs.values() if x > 0]
