@@ -706,6 +706,9 @@ def remove_bytes_parameter_base64(
     Returns:
         None
     """
+    if parameters is None:
+        parameters = {}
+
     for param in parameters:
         request_param = parameters[param]
         if isinstance(request_param, dict) and request_param.get("type") == "bytes":
@@ -889,7 +892,7 @@ def handle_event(event):
             existing_request = db.query_unique(Request, id=event.payload.id)
 
             if existing_request:
-                for field in ("status", "output", "error_class"):
+                for field in ("status", "output", "error_class", "status_updated_at"):
                     setattr(existing_request, field, getattr(event.payload, field))
 
                 try:
